/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.application.slash.converters.impl.enumChoice
import com.kotlindiscord.kord.extensions.commands.converters.impl.channel
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import com.kotlindiscord.kord.extensions.utils.hasPermission
import dev.kord.common.Color
import dev.kord.common.entity.Permission
import net.moonleay.lilJudd.data.tables.TimePlanningChannels
import net.moonleay.lilJudd.data.tables.TimePlanningChannels.channelid
import net.moonleay.lilJudd.data.tables.TimePlanningChannels.serverid
import net.moonleay.lilJudd.extensions.component.EnableOrDisable
import net.moonleay.lilJudd.extensions.component.Feature
import net.moonleay.lilJudd.util.MessageUtil
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insert
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.transactions.transaction

class FeatureManageExtension : Extension() {

    override val name = "feature"

    override suspend fun setup() {
        publicSlashCommand(::FeatureManagerArgs) {
            name = "feature"
            description = "Manage features"

            this.action {
                val u = this.user
                if (!u.asMember(this.guild!!.id).hasPermission(Permission.Administrator)) {
                    this.respond {
                        embeds.add(
                            MessageUtil.getEmbed(
                                Color(0xE0311A),
                                "403: Forbidden",
                                "You cannot edit features, as you don't have the Administratior permission.",
                                u.asUser().username + "#" + u.asUser().discriminator
                            )
                        )
                    }
                    return@action
                }

                if (this.arguments.setStatus == EnableOrDisable.ENABLE) {
                    when (this.arguments.feature) {
                        Feature.TIMEPLANNINGFEATURE -> {
                            val gID = this.guild!!.id.toString()
                            val cID = this.arguments.channel.id.toString()
                            val args = this.arguments
                            var alreadyExists: Boolean = false
                            transaction {
                                val entryExists = TimePlanningChannels.select {
                                    (serverid eq gID) and
                                            (channelid eq cID)
                                }.count() > 0

                                alreadyExists = entryExists
                            }
                            if (!alreadyExists) {
                                transaction {
                                    TimePlanningChannels.insert {
                                        it[serverid] = gID
                                        it[channelid] = cID
                                    } get TimePlanningChannels.id
                                }
                                this.respond {
                                    embeds.add(
                                        MessageUtil.getEmbed(
                                            Color(0x52E01A),
                                            "200: Success",
                                            "The feature was enabled in channel ${args.channel.data.name.value}",
                                            u.asUser().username + "#" + u.asUser().discriminator
                                        )
                                    )
                                }
                                return@action
                            }
                            this.respond {
                                embeds.add(
                                    MessageUtil.getEmbed(
                                        Color(0xE0311A),
                                        "403: Forbidden",
                                        "The feature is already enabled in this channel.",
                                        u.asUser().username + "#" + u.asUser().discriminator
                                    )
                                )
                            }
                        }
                    }
                    return@action
                }
                //Disable
                when (this.arguments.feature) {
                    Feature.TIMEPLANNINGFEATURE -> {
                        val gID = this.guild!!.id.toString()
                        val cID = this.arguments.channel.id.toString()
                        var alreadyExists = false
                        transaction {
                            val entryExists = TimePlanningChannels.select {
                                (serverid eq gID) and
                                        (channelid eq cID)
                            }.count() > 0

                            alreadyExists = entryExists
                        }
                        if (alreadyExists) {
                            transaction {
                                val matchingEntries = TimePlanningChannels.select {
                                    (serverid eq gID) and
                                            (channelid eq cID)
                                }.toList()

                                matchingEntries.forEach { entry ->
                                    TimePlanningChannels.deleteWhere { id eq entry[id] }
                                }
                            }
                            this.respond {
                                embeds.add(
                                    MessageUtil.getEmbed(
                                        Color(0x52E019),
                                        "200: Success",
                                        "The feature was disabled.",
                                        u.asUser().username + "#" + u.asUser().discriminator
                                    )
                                )
                            }
                            return@action
                        }
                        this.respond {
                            embeds.add(
                                MessageUtil.getEmbed(
                                    Color(0xE0311A),
                                    "403: Forbidden",
                                    "The feature is already disabled in this channel.",
                                    u.asUser().username + "#" + u.asUser().discriminator
                                )
                            )
                        }
                    }
                }
            }
        }
    }

    inner class FeatureManagerArgs : Arguments() {

        val feature by enumChoice<Feature> {
            this.name = "feature"
            this.description = "The targeted feature"
            this.typeName = "en_US"
        }

        val setStatus by enumChoice<EnableOrDisable> {
            this.name = "set"
            this.description = "Set enabled or disabled"
            this.typeName = "en_US"
        }

        val channel by channel {
            this.name = "channel"
            this.description = "Target Channel"
        }
    }
}
